// SNF4CGP/OutputProcessor.hpp
// Copyright (C) 2009 ARM Research Labs, LLC
// See www.armresearch.com for more information.
//
// The OutputProcessor feeds responses back to CommuniGate from completed jobs
// and recycles jobs back into the JobPool. This is done in a separate thread so
// that the output stream (cout) is decoupled from the rest of the application.
// If the output stream blocks momentarily then only this thread is blocked while
// all other jobs continue to operate with full steam.

#ifndef IncludedOutputProcessor
#define IncludedOutputProcessor

#include "../CodeDweller/threading.hpp"

class JobPool;                                                                  // Yes there is a JobPool class.
class Job;                                                                      // Yes there is a Job class.

class OutputProcessor : private Thread {                                        // Process job outputs.
  private:

    Mutex ProtectCout;

    JobPool* RecycledJobs_;                                                     // Where we put our recycled jobs.
    JobPool& RecycledJobs();                                                    // Safe access to jobs pool.
    ProductionQueue<Job*> CompletedJobs;                                        // Input queue for jobs to process.

    bool isTimeToStop;                                                          // Flag for when we need to stop.
    void myTask();                                                              // Task for the thread. (my loop).

    void handleJob(Job& J);                                                     // Process a job.

  public:

    OutputProcessor();                                                          // Constructed with the JobPool.
    ~OutputProcessor();                                                         // Destructor to clean things up.

    void init(JobPool& J);                                                      // Start processing.
    void outputJob(Job& J);                                                     // Take this job and ... you know.
    void sendString(const string& S);                                           // Safely send a string to cout.
    void stop();                                                                // Finish off the queue and quit.

    const static ThreadType Type;
    const static ThreadState WaitingForJobs;
    const static ThreadState PostingResponses;
};

#endif
